import QtQuick 2.9
import ScanRenderer 1.0
import AppSettings 1.0

import Qt3D.Core 2.0
import QtQuick.Scene3D 2.0

import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0

import ".."
import "../../controls"

Item {
    id: usimage_area_root

    property bool commandProcessingState

    // The top bar offsets the frame data in the Y direction while displaying
    property real image_offset_y
    // The action bar offsets the frame data in the X direction in Landscape mode
    property real image_offset_x

    readonly property bool gainSliderActive: (pinch_area_1.gain_depth_slider && pinch_area_1.gain_depth_slider.gain_active) || scan_gain_control.applyingGainValue
    readonly property bool depthSliderActive: pinch_area_1.gain_depth_slider && pinch_area_1.gain_depth_slider.depth_active
    readonly property bool tgcActive: pinch_area_1.gain_depth_slider && pinch_area_1.gain_depth_slider.tgc_active

    property Item measurement_group: isRecallMediaModeOn ? recalledMediaView.measurementGroup : measurement_group_scanrenderer
    property alias measurement_caliper_control: measurement_caliper_control
    property alias scanrenderer: scanrenderer
    property alias color_roi_control: color_roi_control
    property alias restrictor_item: ultrasound_image_area_restrictor

    readonly property bool zoomIsActivated: pinch_area_1.zoomActive
    property int measurement_unit: app_settings.measurementUnit
    property string measurement_unit_string: (measurement_unit === AppSettings.CENTIMETER) ? qsTr("cm") : qsTr("mm")

    PinchPanControl {
        id: pinch_area_1
        Accessible.name: "pinch_area"
        anchors.fill: parent
        enabled: !(measurement_group.count > 0) && scan_ui_mgr.providerHasImages
        pan_mouse_area.enabled: scan_ui_mgr.providerHasImages
        gain_depth_slider.enabled: scan_ui_mgr.providerHasImages && !scan_ui_mgr.frozen && !zoomIsActivated
        defaultScale: 1.0/Math.max(scanrenderer.bboxLowerRight.y - scanrenderer.bboxUpperLeft.y, 0.001);
        defaultNormalizedWidth: defaultScale
        defaultNormalizedHeight: defaultScale
        defaultNormalizedX: (1.0 - defaultNormalizedWidth)/2.0 - 0.5
        defaultNormalizedY: (1.0 - defaultNormalizedHeight)/2.0 - 0.5

        onUpdateTGCValues: {
            var p = mapToItem(tgc_control_view, 0, y)
            tgc_control_view.updateSliderAt(p.y, delta_x)
        }
        onCommitTGCValues: {
            var p = mapToItem(tgc_control_view, 0, y)
            tgc_control_view.releaseSliderAt(p.y)
        }

        function doClicked(mouse) {
            if (measurement_group.count > 0) return
            if(pinch_area_1.panMovement) return

            scan_ui_mgr.toggleRunState()
        }

        pan_mouse_area.onClicked: {
            //If isRecallMediaModeOn is true , means gallery images are being viewed. So don't propagate click events.
            //If in future, zoom-in/zoom-out/click on image are allowed on recalledMediaView, we may need to reconsider below if condition
           if(!isRecallMediaModeOn)
                doClicked(mouse)
        }
        Connections {
            target: scan_ui_mgr
            onFrozenChanged: {
                // If we are changing back to live and this is not due to video save, then reset the zoom-pan params.
                if (!frozen && !media_manager.videoProcessingState) {
                    pinch_area_1.resetToDefaultProperties()
                }
            }
        }

        Connections {
            target: scan_ui_mgr

            onPresetChanged: {
                pinch_area_1.resetToDefaultProperties()
            }
            onCommandStatusCompleted: {
                commandProcessingState = false
            }

            onCommandStatusProcessing: {
                commandProcessingState = true
            }

            onCommandStatusStopProcessing: {
                commandProcessingState = false
            }
        }

        Connections {
            target: command_error_mgr
            onCommandStatusFailed: {
                // TODO: something wrong with acquistion, discuss workflow, add dialog to restart app
                commandProcessingState = false
                app.showBanner(qsTr("Command failed. Restart probe and application."))
            }
            onCommandStatusTimeOut: {
                // TODO: something wrong with acquistion, discuss workflow, add dialog to restart app
                commandProcessingState = false
                app.showBanner(qsTr("Command timed out. Probe connection failed."))
            }
        }

        Connections {
            target: exam_manager
            /* explicitly reset the zoom and pan when we change exam id.
             * Currently it is not required since we always exit recall view into live mode, hence reset of zoom and pan
             * is implicit. But we should depend on implicit reset which might change when workflow changes.
             */
            onViewedExamParamsChanged: {
                pinch_area_1.resetToDefaultProperties()
            }
        }
    }

    // This item moves the coordinate system to the center of the screen
    // so that the panning and zooming is consistent when the orientation
    // of the scree changes
    Item {
        id: renderer_overlays
        objectName: "rendereroverlays"
        x: usimage_area_root.width/2.0
        y: usimage_area_root.height/2.0
        // In order for the mouse areas to work, this item
        // needs proper width and height
        width: usimage_area_root.width
        height: usimage_area_root.height
        z: 2
        visible: !isRecallMediaModeOn

        ScanRenderer {
            id: scanrenderer
            visible: !isRecallMediaModeOn
            objectName: "scanrenderer"
            property bool frame_data: true

            x: pinch_area_1.normalizedX*parent.height
            y: pinch_area_1.normalizedY*parent.height

            width: pinch_area_1.normalizedWidth*parent.height
            height: pinch_area_1.normalizedHeight*parent.height

            smooth: false
            antialiasing: false
            z:1

            onSignalTissueGeometryChanged: {
                scan_depth_marker_view_1.slotUpdateDepthMarkerGeometry(render_geometry, image_2d_geom)
                measurement_group_scanrenderer.slotUpdateRenderGeometry(render_geometry)
                scan_orientation_marker.slotUpdateTissueGeometry(render_geometry, image_2d_geom)
                scan_centerline_marker.slotUpdateTissueGeometry(render_geometry, image_2d_geom)
                scan_focus_marker.slotUpdateTissueGeometry(render_geometry, image_2d_geom)
                scan_depth_preview_view.slotUpdateTissueGeometry(render_geometry, image_2d_geom)
                tgc_control_view.updateGeometry(render_geometry, image_2d_geom)
                scan_ui_mgr.slotTissueGeometryChanged(image_2d_geom)
            }
            onSignalFlowGeometryChanged: {
                color_roi_1.slotUpdateColorROIGeometry(render_geometry, tissue_image_2d_geom, flow_image_2d_geom)
                color_roi_control.slotUpdateColorROIGeometry(render_geometry, tissue_image_2d_geom, flow_image_2d_geom)
                scan_depth_preview_view.slotUpdateFlowGeometry(flow_image_2d_geom);
            }
            onSignalFlowColorMapChanged: {
                scan_color_bar.slotUpdateColorBarImage(image)
            }
            onSignalTextureReady: {
                scan_depth_preview_view.texture_item.updateTexture(id, size)
            }
        }

        ScanColorROIView {
            id: color_roi_1
            objectName: "scancolorroi"
            visible: !isRecallMediaModeOn && scan_ui_mgr.flowEnabled
            enabled: false
            anchors.fill: scanrenderer
            opacity: 1.0
            border_color: theme.melodyGray
            property bool save_data: true
            z:2
        }

        ScanColorROIControlsView {
            id: color_roi_control
            objectName: "scancolorroicontrol"
            visible: !isRecallMediaModeOn && scan_ui_mgr.flowEnabled
            anchors.fill: scanrenderer
            border_color: isActive ? theme.sexyBlueColor : theme.melodyGray
            bottomOffset: isLandscape ? 0.0 : (melody_action_bar.height/us_image_view_area.height)
            z:3
            onClicked: pinch_area_1.doClicked(mouse)
        }

        ScanDepthMarkerView {
            id: scan_depth_marker_view_1
            objectName: "scandepthmarker"
            visible: !isRecallMediaModeOn

            anchors.fill: scanrenderer
            restrictor: ultrasound_image_area_restrictor
            property bool save_data: true

            opacity: 1.0
            lineWidth: 2.7 * scaleFactor
            largeTickWidth: 16 * scaleFactor
            smallTickWidth: 7 * scaleFactor
            lastTickWidth: 21 * scaleFactor
            tickColor: theme.sexyBlueColor
            textColor: "white"
            z:4

            preferredAnchorPoint: Qt.point(width, scanrenderer.bboxUpperLeft.y*height)
            restrictedAnchorPoint: Qt.point(restrictor.width, 0)
        }
	
        ScanFocusMarkerView {
            id: scan_focus_marker
            objectName: "scanfocusmarker"
            visible: !isRecallMediaModeOn && app_settings.bmodeMarkerEnabled

            anchors.fill: scanrenderer
            restrictor: ultrasound_image_area_restrictor
            property bool save_data: true

            opacity: 1.0
            z:5

            preferredAnchorPoint: Qt.point(width, scanrenderer.bboxUpperLeft.y*height)
            restrictedAnchorPoint: Qt.point(restrictor.width, 0)
        }
	
        ScanOrientationMarkerView {
            id: scan_orientation_marker
            objectName: "scanorientationmarker"
            property bool save_data: true
            z: 6
            visible: !isRecallMediaModeOn

            Behavior on opacity {
                NumberAnimation { duration: theme.shortAnimationDuration; easing.type: Easing.InOutQuad }
            }

            anchors.fill: scanrenderer
            restrictor: ultrasound_image_area_restrictor
        }

        ScanCenterlineMarkerView {
            id: scan_centerline_marker
            objectName: "scancenterlinemarker"
            property bool save_data: true
            z: 7
            visible: !isRecallMediaModeOn

            Behavior on opacity {
                NumberAnimation { duration: theme.shortAnimationDuration; easing.type: Easing.InOutQuad }
            }

            anchors.fill: scanrenderer
        }

        ScanColorBarView {
            id: scan_color_bar
            objectName: "scancolorbar"
            z: 8
            visible: !isRecallMediaModeOn
            property bool save_data: true

            Behavior on opacity {
                NumberAnimation { duration: theme.shortAnimationDuration; easing.type: Easing.InOutQuad }
            }

            anchors.fill: scanrenderer
            restrictor: ultrasound_image_area_restrictor

            preferredAnchorPoint: Qt.point(width, scanrenderer.bboxUpperLeft.y*height)
            restrictedAnchorPoint: Qt.point(restrictor.width, 0)
        }

        ScanDepthPreviewView {
            id: scan_depth_preview_view
            objectName: "scandepthpreview"
            depth_active: depthSliderActive
            visible: !isRecallMediaModeOn && depth_active
            enabled: true
            anchors.fill: scanrenderer
            bottomOffset: isLandscape ? 0.0 : (melody_action_bar.height/us_image_view_area.height)
            property bool save_data: false
            z: 9
        }

        TGCControlView {
            id: tgc_control_view
            objectName: "tgccontrol"
            anchors {
                top: scanrenderer.top
                bottom: scanrenderer.bottom
                left: restrictor.left
                right: restrictor.right
            }
            restrictor: interactive_ultrasound_area_restrictor
            z: 9
            active: usimage_area_root.tgcActive
        }

        Item {
            id: interactive_ultrasound_area_restrictor
            x: Math.max(-usimage_area_root.width/2.0, -scanrenderer.width/2.0)
            y: -usimage_area_root.height/2.0
            width: Math.min(ultrasound_image_area_restrictor.width, scanrenderer.width)
            height: isLandscape ? ultrasound_image_area_restrictor.height : ultrasound_image_area_restrictor.height - melody_action_bar.height
            z: 100
        }

        MeasurementCaliperControlView {
            id: measurement_caliper_control
            measurement_group: measurement_group_scanrenderer
            anchors.fill: scanrenderer
            z: 10
            movement_restrictor: interactive_ultrasound_area_restrictor
        }

        MeasurementGroup {
            id: measurement_group_scanrenderer
            objectName: "measurementgroupscanrenderer"
            property bool save_data: true
            enabled: requested_shown && scan_ui_mgr.frozen && !app.isRecallMediaModeOn
            display_restrictor: ultrasound_image_area_restrictor
            anchors.fill: scanrenderer
            line_scale: pinch_area_1.currentScale/Math.max(pinch_area_1.defaultScale, 0.0001)
            z: 11
        }

        ScanInfoOverlay {
            anchors.fill: parent
            z: 12
        }

        ScanGainControlView {
            id: scan_gain_control_view
            objectName: "scangaincontrol"
            enabled: true
            anchors.fill: scanrenderer
            scanrenderer_bbox_upper_left_y: scanrenderer.bboxUpperLeft.y
            visible: !scan_ui_mgr.frozen && gainSliderActive
            property bool save_data: false
            z: 13
        }

        OrganLabel {

            m_organ: "LIVER"

            m_x: pinch_area_1.zoomActive ? -usimage_area_root.width : (((scanrenderer.width/1000.0) * inference_data.liver_x) - (scanrenderer.width/2.0))
            m_y: pinch_area_1.zoomActive ? -usimage_area_root.height : (((scanrenderer.height/1000.0) * inference_data.liver_y) - (scanrenderer.height/2.0)) - (((pinch_area_1.defaultScale - 1.0) *usimage_area_root.image_offset_y)/2.0)

            z: scanrenderer.z

            m_fontfactor: calculateFontSize()

            function calculateFontSize()
            {
                var defaultfontSize_HW = 10.0
                var defaultfontSize_WH = 10.0
                if(Qt.platform.os == "android")
                {
                    defaultfontSize_HW = 20.0
                    defaultfontSize_WH = 16.0
                }
                else if(Qt.platform.os == "ios")
                {
                    defaultfontSize_HW = 9.0
                    defaultfontSize_WH = 6.0
                }

                var ratio_HW = usimage_area_root.height/usimage_area_root.width
                var ratio_WH = usimage_area_root.width/usimage_area_root.height
                if( ratio_HW < ratio_WH)
                    return defaultfontSize_WH * ratio_WH;
                else
                    return defaultfontSize_HW * ratio_HW;
            }

        }

        OrganLabel {
            m_organ: "KIDNEY"

            m_x: pinch_area_1.zoomActive ? -usimage_area_root.width : ((scanrenderer.width/1000.0 * inference_data.kidney_x) - scanrenderer.width/2.0)
            m_y: pinch_area_1.zoomActive ? -usimage_area_root.height : ((scanrenderer.height/1000.0 * inference_data.kidney_y) - (scanrenderer.height/2.0)) - (((pinch_area_1.defaultScale - 1.0) *usimage_area_root.image_offset_y)/2.0)

            z: scanrenderer.z

            m_fontfactor: calculateFontSize()

            function calculateFontSize()
            {
                var defaultfontSize_HW = 10.0
                var defaultfontSize_WH = 10.0
                if(Qt.platform.os == "android")
                {
                    defaultfontSize_HW = 20.0
                    defaultfontSize_WH = 16.0
                }
                else if(Qt.platform.os == "ios")
                {
                    defaultfontSize_HW = 9.0
                    defaultfontSize_WH = 6.0
                }

                var ratio_HW = usimage_area_root.height/usimage_area_root.width
                var ratio_WH = usimage_area_root.width/usimage_area_root.height
                if( ratio_HW < ratio_WH)
                    return defaultfontSize_WH * ratio_WH;
                else
                    return defaultfontSize_HW * ratio_HW;
            }

        }

        OrganLabel {
            m_organ: "GB"

            m_x: pinch_area_1.zoomActive ? -usimage_area_root.width : ((scanrenderer.width/1000.0 * inference_data.gb_x) - scanrenderer.width/2.0)
            m_y: pinch_area_1.zoomActive ? -usimage_area_root.height : ((scanrenderer.height/1000.0 * inference_data.gb_y) - (scanrenderer.height/2.0)) - (((pinch_area_1.defaultScale - 1.0) *usimage_area_root.image_offset_y)/2.0)

            z: scanrenderer.z

            m_fontfactor: calculateFontSize()

            function calculateFontSize()
            {
                var defaultfontSize_HW = 10.0
                var defaultfontSize_WH = 10.0
                if(Qt.platform.os == "android")
                {
                    defaultfontSize_HW = 20.0
                    defaultfontSize_WH = 16.0
                }
                else if(Qt.platform.os == "ios")
                {
                    defaultfontSize_HW = 9.0
                    defaultfontSize_WH = 6.0
                }

                var ratio_HW = usimage_area_root.height/usimage_area_root.width
                var ratio_WH = usimage_area_root.width/usimage_area_root.height
                if( ratio_HW < ratio_WH)
                    return defaultfontSize_WH * ratio_WH;
                else
                    return defaultfontSize_HW * ratio_HW;
            }

        }
        

        

//        Rtsc {
//            x_cordinate: 0.0 - usimage_area_root.width/2.0
//            y_cordinate: 0.0 - usimage_area_root.height/2.0
//            m_currentText: inference_data.GuidanceText
//            m_organStateImage1: inference_data.iconStatusStepOne
//            m_organStateImage2: inference_data.iconStatusStepTwo
//            m_organStateImage3: inference_data.iconStatusStepThree
//            m_progressStatebtw1and2: inference_data.iconStepLineOne
//            m_progressStatebtw2and3: inference_data.iconStepLineTwo
//            m_probePosition: inference_data.organ_probe_state_img
//            m_probeAction: inference_data.guidance_state_img
//            z: 23
//        }
    }

    Item {
        id: ultrasound_image_area_restrictor
        visible: !isRecallMediaModeOn
        height: parent.height
        width: parent.width
    }

    RecalledMediaView {
        id: recalledMediaView
        objectName: "recallrenderer"
        z: 9
        visible: isRecallMediaModeOn
        anchors.fill: usimage_area_root
    }

    FilterControlView {
        id: filter_control_view
        visible: app_settings.filterUiEnabled
        anchors {
            top: parent.top
            topMargin: theme.defaultMargin
            left: parent.left
            leftMargin: theme.defaultMargin
        }
        width:500*scaleFactor
        height:parent.height
        z: 20
    }


///////////////////////

Item {
    id: root

        x: 0
        y: 387
        z: 14
        width: parent.width - 12
        height: parent.height - 420
        visible: true

        property double xRotSlider: 0

        Text {
            text: "Click me!"
            color: "yellow"
            anchors.top: parent.top
            //anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
//            anchors {
//                top: parent.top
//                topMargin: theme.defaultMargin
//                left: parent.left
//                leftMargin: theme.defaultMargin
//            }

            MouseArea {
                anchors.fill: parent
                onClicked: animation.start()
            }
        }

        Rectangle {
            id: scene
            anchors.fill: parent
            anchors.topMargin: 24
            //anchors.margins: 10
            color: "gray"
            opacity:0.9

            transform: Rotation {
                id: sceneRotation
                axis.x: 1
                axis.y: 0
                axis.z: 0
                origin.x: scene.width / 2
                origin.y: scene.height / 2
            }

            Scene3D {
                id: scene3d
                anchors.fill: parent
                anchors.margins: 10
                focus: true
                aspects: ["input", "logic"]
                cameraAspectRatioMode: Scene3D.AutomaticAspectRatio

                AnimatedEntity {}
            }
        }

        //! [1]
        RowLayout {
            id: controlLayout
            spacing: 5
            x: 12
            y: parent.height - 100
            width: parent.width - (2 * x)
            height: 100
            visible: true
            //! [1]

            Label {
                id: xRotLabel
                Layout.alignment: Qt.AlignRight
                Layout.fillWidth: false
                text: "X-axis:"
            }

            //! [2]
            Slider {
                id: xSlider
                Layout.alignment: Qt.AlignLeft
                Layout.fillWidth: true
                minimumValue: 0;
                maximumValue: 360;
                //! [4]
                onValueChanged: root.xRotSlider = value;
                //! [4]
            }
            //! [2]

//            Label {
//                id: yRotLabel
//                Layout.alignment: Qt.AlignRight
//                Layout.fillWidth: false
//                text: "Y-axis:"
//            }

//            Slider {
//                id: ySlider
//                Layout.alignment: Qt.AlignLeft
//                Layout.fillWidth: true
//                minimumValue: 0;
//                maximumValue: 360;
//                //onValueChanged: canvas3d.yRotSlider = value;
//            }

//            Label {
//                id: zRotLabel
//                Layout.alignment: Qt.AlignRight
//                Layout.fillWidth: false
//                text: "Z-axis:"
//            }

//            Slider {
//                id: zSlider
//                Layout.alignment: Qt.AlignLeft
//                Layout.fillWidth: true
//                minimumValue: 0;
//                maximumValue: 360;
//                //onValueChanged: canvas3d.zRotSlider = value;
//            }
        }
}
///////////////////

}

