

#include <QGuiApplication>
#include <QQuickView>
#include <QtQml>
#include <QQmlContext>
#include <QOpenGLContext>
#include <QQmlApplicationEngine>

#include <iostream>


int main(int argc, char **argv)
{

    QSurfaceFormat format;

    if (QOpenGLContext::openGLModuleType() == QOpenGLContext::LibGL)
    { // OpenGL
        format.setRenderableType(QSurfaceFormat::OpenGL);
        format.setVersion(4, 3);
        format.setProfile(QSurfaceFormat::CoreProfile);
        format.setSamples(4);
    }
    else if (QOpenGLContext::openGLModuleType() == QOpenGLContext::LibGLES)
    { // OpenGLES??
        format.setRenderableType(QSurfaceFormat::OpenGLES);
        format.setVersion(3, 0);
        format.setSamples(0);
    }
    else
    {
        std::cout << "Error : QOpenGLContext cannot identify openGL Module" << std::endl;
        exit(0);
    }
    format.setAlphaBufferSize(0);
    format.setDepthBufferSize(0);
    format.setStencilBufferSize(0);
    format.setSwapBehavior(QSurfaceFormat::TripleBuffer);
    format.setSwapInterval(0); // Full speed rendering
    QSurfaceFormat::setDefaultFormat(format);

#if defined(Q_OS_MAC)
    const QString envmapFormat = QLatin1String("-16f");
#else
    const QString envmapFormat = QLatin1String("");
#endif

    QGuiApplication app(argc, argv);


    QQuickView view;
    view.engine()->rootContext()->setContextProperty(QLatin1String("_envmapFormat"), envmapFormat);
    view.engine()->rootContext()->setContextProperty("_window", &view);

    view.resize(1280, 820);
    view.setResizeMode(QQuickView::SizeRootObjectToView);
    view.setSource(QUrl("qrc:/main.qml"));
    view.show();

    return app.exec();

}
